#----------------------------------------------------------------#
# This file includes R code for implementing the proposed BLAST  #
# design when 6 disease types are evaluated; it is written in a  #
# way to be run on clusters.                                     #
# Two functions are included:                                    #
# 1) trajectories() for simulating the binary and longitudinal   #
#    data;                                                       #
# 2) LatentBasket() for generating operating characteristics;    #
#----------------------------------------------------------------#





#--------------------1. Parallel cluster setting-----------------#

library(foreach)
library(doParallel)
jEOM = F
raw_PE <- switch(jEOM+1, F = "localhost.nautilus.mdanderson.edu", T = system("cat $PE_HOSTFILE",intern=T))  # or use $HOSTNAME ,`hostname -s`
cat('raw_PE is',raw_PE)
nodes_name <- sub("^([\\w-]+)\\..*$","\\1",raw_PE,perl=T)
nodes_slots <- switch(jEOM+1, F = as.numeric(system("cat /proc/cpuinfo | grep processor|wc -l",intern=T)),T = as.numeric( sub("^([\\w-]+)\\..* (\\d+?) .*$","\\2",raw_PE,perl=T) ) )

socketHosts_name <- rep(nodes_name,times=24)
cl <- makePSOCKcluster(socketHosts_name,homogeneous=TRUE)
clusterCall(cl, function() Sys.info()[c("nodename","machine")])
registerDoParallel(cl)
clusterExport(cl=cl,ls())

#-------------------Main functions go below -----------------------#
setwd("YourFilePath")
#---------------1. Trajectory function--------------------#
trajectories<-function(t,shape, ngroup,  groupind, npatient, sigma.group, sigma.pt, sigma.error,betaY, p.true, plot=FALSE){
  #set.seed(123)
  curve<-array(NA,dim=c(npatient,length(t),ngroup))
  data<-NULL
  bdata<-NULL
  csize<-rep(npatient,ngroup) # for binary data
  for (i in 1:ngroup){
    if (shape[groupind[i]]=="hump"){
      cluster.shape<-18-12*exp(-6*(t+0.05))
    } else if (shape[groupind[i]]=="decay"){
      cluster.shape<-5*exp(-t^(0.5))+4
    } else if (shape[groupind[i]]=="constant"){
      cluster.shape<-18-12*exp(-6*0.05)
    } else if (shape[groupind[i]]=="rise"){
      cluster.shape<-8.86+8/(1+exp(-8*(t-0.5)))
    }
    group.error <- rnorm(1, 0, sigma.group)
    pt.error<-rnorm(npatient,0, sigma.pt)
    measure.error<-rnorm(npatient*length(t), 0, sigma.error)
    y.temp<-rbinom(csize[i], 1, p.true[groupind[i]])
    index.temp <-rep(groupind[i], npatient)
    curve[,,i]<-matrix(rep(cluster.shape+group.error,npatient),nrow=npatient, ncol=length(cluster.shape),byrow=T) # curve in wide format for plotting
    curve[,,i]<-curve[,,i]+pt.error+matrix(measure.error, nrow=npatient,ncol=length(t),byrow=T)+betaY*y.temp
    temp<-cbind(rep(cluster.shape+group.error, npatient)+measure.error+rep(pt.error,each=length(t))+betaY*rep(y.temp, each=length(t)), 
                rep(seq(npatient*i-(npatient-1),npatient*i,1),each=length(t)),rep(t,npatient),rep(groupind[i],npatient*length(t)))
    data<-rbind(data,temp) # data in long format for modeling
    
    
    bdata<-rbind(bdata, cbind(y.temp, index.temp))
  }
  colnames(data)<-c("z", "ID", "t", "Group")
  
  if (plot==TRUE){
    par(mfrow=c(2,3))
    for (j in 1:ngroup){
      plot(t,curve[,,j][1,], type="l", xlab="Time", ylab="",ylim=c(0,20))
      for (i in (2: nrow(curve[,,j]))){
        lines(t,curve[,,j][i,])
      }
      lines(t,18-12*exp(-6*(t+0.05)),col="red",lty=2, lwd=2)
      #lines(t,-1+5*exp(-t^(0.5))+3,col="blue",lty=2, lwd=2)
      lines(t,rep(18-12*exp(-6*0.05),length(t)), col="orange",lty=2,lwd=2)
      #lines(t,5+10/(1+exp(-7*(t-0.5))), col="green", lty=2, lwd=2)
    }
  }
  
  results<-list(curve=curve, data=data, bdata=bdata)
  results
}


#--------------------------2. Joint Modeling MCMC------------------------------------#



  
source("oneclusterMCMC_v7.R")
source("twoclusterMCMC_v7.R")





#------------------------3. Basket trial conduction -----------------------------------#
#------------------------------------------------------#
# To build the structure of phase II basket trial,     #
# using proposed model to estimate subgroup effects.   #
#------------------------------------------------------# 


LatentBasket<-function( cohortsize, ngroup,shape.true, p.true, p.null, p.target, nknot, mcsim, nburnin, ci.true){
  
  
  stop<-numeric(ngroup)
  binary.data<-NULL
  biomarker.data<-NULL
  group.num<-ngroup    # number of groups, initial value = ngroup
  group.ind<-seq(1,ngroup,1)       # group indicator, initial value = 1:length(ngroup)
  interim.instruct<-c(rep(TRUE,length(cohortsize)-1),FALSE)
  
  K.trial<-c(1,2)
  k.index<-c(rep(1, 3+nknot), rep(2,2*(3+nknot)))              # for spline parameter selection
  dic.sel<-numeric(length(K.trial))
  stop.ind<-matrix(NA, nrow=length(K.trial),ncol=ngroup)
  gonogo<-numeric(length(K.trial))
  grp.effect<-matrix(NA, nrow=length(K.trial),ncol=ngroup)  # store the results from 3 separate cluster analysis
  grp.p<-matrix(NA, nrow=length(K.trial),ncol=ngroup)       # store the p.posterior.mean from 3 separate cluster analysis
  grp.final<-numeric(ngroup)                                # store the result for final group response measure for this trial
  grp.size<-numeric(ngroup)
  theta.Kpost<-matrix(0,nrow=length(K.trial), ncol=2)
  alpha.final<-matrix(0,nrow=length(K.trial), ncol=2*(3+nknot))                                         # store the spline parameters from 3 separate cluster analysis
  # interim 1
  for (H in 1:length(cohortsize)){
    # generated longitudinal data
    set.seed(100+q+H) # change
    datgen<-trajectories(t=seq(0,1,1/7),shape=shape.true, ngroup=group.num, groupind=group.ind, npatient=cohortsize[H], 
                         sigma.group=sqrt(4), sigma.pt=sqrt(4), sigma.error=sqrt(1.5), betaY=1, p.true=p.true,plot=FALSE)
    z.temp<-datgen$data 
    
    if (H!=1){
      biomarker.data<-rbind(biomarker.data, z.temp)                             # combine with previous interim data
      biomarker.data<-biomarker.data[order(biomarker.data[,4]),]                # sort by group indicator
      biomarker.data[,2]<-rep(seq(1, nrow(biomarker.data)/8,1), each=8)       # update patient ID
    } else{
      biomarker.data<-z.temp
    }
    
    
    # generated binary data
    ydat.temp<-datgen$bdata
 
    if (H!=1){
      binary.data<-rbind(binary.data, ydat.temp)
      binary.data<-binary.data[order(binary.data[,2]),]
    } else{
      binary.data<-ydat.temp
    }
    
    
    
    # MCMC, model fit
    #Sys.time()
    one<-OneCluster(longdata=biomarker.data,outcomedata=binary.data, nknot=4, mcsim=8000, nburnin=4000,ncluster=1, p.null=0.2, p.target=0.3,interim=interim.instruct[H],
                      stop.input=stop)
    #Sys.time()  
    two<-TwoCluster(longdata=biomarker.data,outcomedata=binary.data, nknot=4, mcsim=8000, nburnin=4000,ncluster=2, p.null=0.2, p.target=0.3,ci.true=ci.true, interim=interim.instruct[H],
                    stop.input=stop)  
    #Sys.time()  
    dic.sel<-c(one$DIC, two$DIC); stop.ind<-rbind(one$stop.output, two$stop.output) ; gonogo<-c(one$instruction,two$instruction)
    grp.effect<-rbind(one$trteff.prob, two$trteff.prob); grp.p<-rbind(one$outcome.p.mean, two$outcome.p.mean)
    
   
    # update group.num, group.ind, stop
    if (abs(dic.sel[1]-dic.sel[2])<=3){
      model.index<-1
    } else{
      model.index<-which(dic.sel==min(dic.sel))
    }
    stop<-stop.ind[model.index,]                                 # update stop vector
    group.num<-length(which(stop==0))              # update number of groups remain
    group.ind<-which(stop==0)                      # update group indicator of those remain
    if (gonogo[model.index]!="continue"){
      grp.final<-grp.effect[model.index,]   # final group effectiveness measure
      grp.size<-as.vector(table(binary.data[,2]))            # final sample size for each subgrp
      grp.p.mean<-grp.p[model.index,]       # final group.p.mean 
      K.sel<-K.trial[model.index]           # final selected number of clusters
      dic<-dic.sel[model.index]
      if (K.sel==2){
        spline.final<-two$alpha.post
       
        theta.final<-two$theta.Kmean
        tau_mi<-two$tau_mi
        v.prob<-two$v.prob
        ci<-two$ci
        beta<-mean(two$betaY)
        betaCI<-quantile(two$betaY, c(0.25, 0.5, 0.975))
      } else {
        spline.final<-one$alpha.post
        
        theta.final<-one$theta.Kmean
        tau_mi<-one$tau_mi
        v.prob<-0
        ci<-0
        beta<-mean(one$betaY)
        betaCI<-quantile(one$betaY, c(0.25, 0.5, 0.975))
      }
      
      break
    }
    
    
    
  }
  
  result1<-c(grp.final,grp.size,grp.p.mean,  dic.sel, K.sel, theta.final, beta, betaCI)
  results<-list(result1, spline.final, ci)
  results
}






ngroup<-6
p.null<-rep(0.2, ngroup)
p.target<-rep(0.3, ngroup)
# shapeA - scenario 1
p.true<- c(rep(0.3,4), rep(0.2,2))
nknot<-4
mcsim<-8000
nburnin<-4000
cohortsize=c(10,5,5,5)
shape.true<-c(rep("hump", 4), rep("constant",2))
ci.true<-rbind(c(0,1),c(0,1),c(0,1),c(0,1),c(1,0),c(1,0))






# scenario 1
joint_A_2a<-foreach(q=1:240, .combine=rbind)  %dopar%  
{
  LatentBasket(cohortsize=c(10,5,5,5), ngroup=6,shape.true=shape.true, p.true=p.true, p.null=p.null, p.target=p.target, nknot=4, mcsim=8000, nburnin=4000, ci.true=ci.true)
} 

save(joint_A_2a,file="LatentBasket_A_S2_240.RData")


